<?php
/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Sascha Nitsch (grumpydeveloper)
 **/

namespace Federator\Data\ActivityPub\Common;

class Note extends APObject
{
    /**
     * sender
     *
     * @var string $sender
     */
    private $sender = "";

    /**
     * receiver
     *
     * @var string $receiver
     */
    private $receiver = "";

    public function __construct()
    {
        parent::__construct("Note");
    }

    /**
     * set sender
     *
     * @param string $sender note sender
     * @return Note current instance
     */
    public function setSender(string $sender)
    {
        $this->sender = $sender;
        return $this;
    }

    /**
     * get sender
     *
     * @return string sender
     */
    public function getSender()
    {
        return $this->sender;
    }

    /**
     * set receiver
     *
     * @param string $receiver note receiver
     * @return Note current instance
     */
    public function setReceiver(string $receiver)
    {
        $this->receiver = $receiver;
        return $this;
    }

    /**
     * convert internal state to php array
     * @return array<string,mixed>
     */
    public function toObject()
    {
        $return = parent::toObject();
        if ($this->sender !== "") {
            $return['sender'] = $this->sender;
        }
        if ($this->receiver !== "") {
            $return['receiver'] = $this->receiver;
        }
        return $return;
    }

    /**
     * create object from json
     * @param mixed $json input json
     * @return bool true on success
     */
    public function fromJson($json)
    {
        if (!parent::fromJson($json)) {
            return false;
        }
        $this->receiver = "";
        return true;
    }
}
