<?php
/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Sascha Nitsch (grumpydeveloper)
 **/

namespace Federator\Data\ActivityPub\Common;

class OrderedCollectionPage extends OrderedCollection
{
    private string $next = '';
    private string $prev = '';
    private string $partOf = '';


    public function __construct()
    {
        parent::__construct();
        parent::addContext('https://www.w3.org/ns/activitystreams');
    }

    /**
     * convert internal state to php array
     *
     * @return array<string,mixed>
     */
    public function toObject()
    {
        $return = parent::toObject();
        if ($this->next !== '') {
            $return['next'] = $this->next;
        }
        if ($this->prev !== '') {
            $return['prev'] = $this->prev;
        }
        if ($this->partOf !== '') {
            $return['partOf'] = $this->partOf;
        }
        $return['type'] = 'OrderedCollectionPage';
        return $return;
    }

    /**
     * create object from json
     *
     * @param array<string,mixed> $json input json
     * @return bool true on success
     */
    public function fromJson($json)
    {
        return parent::fromJson($json);
    }

    /**
     * set next url
     *
     * @param string $url new next URL
     * @return void
     */
    public function setNext($url)
    {
        $this->next = $url;
    }

    /**
     * set prev url
     *
     * @param string $url new prev URL
     * @return void
     */
    public function setPrev($url)
    {
        $this->prev = $url;
    }

    public function setPartOf(string $url) : void
    {
        $this->partOf = $url;
    }
}
