<?php
/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Sascha Nitsch (grumpydeveloper)
 **/

namespace Federator\Data;

/**
 * storage class for statistics
 */
class Stats
{
    /**
     * user count on the external system
     *
     * @var int $userCount
     */
    public $userCount;

    /**
     * post count on the external system
     *
     * @var int $postCount
     */
    public $postCount;

    /**
     * comment count on the external system
     *
     * @var int $commentCount
     */
    public $commentCount;

    /**
     * create new user object from json string
     *
     * @param string $input input string
     * @return Stats
     */
    public static function createFromJson($input)
    {
        $stats = new Stats();
        $data = json_decode($input, true);
        if ($data === null) {
            return $stats;
        }
        $stats->userCount = $data['userCount'];
        $stats->postCount = $data['postCount'];
        $stats->commentCount = $data['commentCount'];
        return $stats;
    }

    /**
     * convert internal data to json string
     *
     * @return string
     */
    public function toJson()
    {
        $data = [
          'userCount' => $this->userCount,
          'postCount' => $this->postCount,
          'commentCount' => $this->commentCount
        ];
        return json_encode($data) | '';
    }
}
