<?php
/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Sascha Nitsch (grumpydeveloper)
 **/

namespace Federator\Data;

/**
 * storage class for user attributes
 */
class User
{
    /**
     * user id on the external system
     *
     * @var string $externalid
     */
    public $externalid;

    /**
     * icon media type
     *
     * @var string $iconMediaType
     */
    public $iconMediaType;

    /**
     * icon url
     *
     * @var string $iconURL
     */
    public $iconURL;

    /**
     * user id
     *
     * @var string $id
     */
    public $id;

    /**
     * image media type
     *
     * @var string $imageMediaType
     */
    public $imageMediaType;

    /**
     * image url
     *
     * @var string $imageURL
     */
    public $imageURL;

    /* @var string user language */
    //public $lang;

    /**
     * user name
     *
     * @var string $name
     */
    public $name = '';
    /**
     * user permissions
     *
     * @var array<string> $permissions
     * @todo convert to enum
     */
    public $permissions = [];

    /**
     * user public key
     *
     * @var string $publicKey
     */
    public $publicKey;

    /**
     * registered unix timestamp
     *
     * @var int $registered
     */
    public $registered = 0;

    /**
     * session id
     *
     * @var string $session
     * */
    public $session = '';

    /**
     * summary for user/profile
     *
     * @var string $summary
     */
    public $summary = '';

    /**
     * type of user (user/group)
     *
     * @var string $type
     */
    public $type = 'group';

    /**
     * create new user object from json string
     *
     * @param string $input input string
     * @return User|false
     */
    public static function createFromJson($input)
    {
        $data = json_decode($input, true);
        if ($data === null) {
            return false;
        }
        $user = new User();
        $user->iconMediaType = $data['iconMediaType'];
        $user->iconURL = $data['iconURL'];
        $user->id = $data['id'];
        $user->imageMediaType = $data['imageMediaType'];
        $user->imageURL = $data['imageURL'];
        $user->externalid = $data['externalid'];
        $user->name = $data['name'];
        /// TODO: replace with enums
        $user->permissions = $data['permissions'];
        $user->publicKey = $data['publicKey'];
        $user->summary = $data['summary'];
        $user->type = $data['type'];
        return $user;
    }

    /**
     * check if use has asked permission
     * @param string $p
     *        permission to check
     *
     * @return bool true if user has permission, false if not
     * @todo convert to ENUM
     */
    public function hasPermission(string $p)
    {
        return in_array($p, $this->permissions, false);
    }

    /**
     * convert internal data to json string
     *
     * @return string
     */
    public function toJson()
    {
        $data = [
            'iconMediaType' => $this->iconMediaType,
            'iconURL' => $this->iconURL,
            'id' => $this->id,
            'imageMediaType' => $this->iconMediaType,
            'imageURL' => $this->iconURL,
            'externalid' => $this->externalid,
            'name' => $this->name,
            'permissions' => $this->permissions,
            'publicKey' => $this->publicKey,
            'summary' => $this->summary,
            'type' => $this->type
        ];
        return json_encode($data) | '';
    }
}
