<?php
/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Yannis Vogel (vogeldevelopment)
 **/

namespace Federator\DIO;

/**
 * IO functions related to followers
 */
class Followers
{

    /**
     * get followers of user
     *
     * @param \mysqli $dbh @unused-param
     *          database handle
     * @param string $id
     *          user id
     * @param \Federator\Connector\Connector $connector
     *          connector to fetch use with
     * @param \Federator\Cache\Cache|null $cache
     *          optional caching service
     * @return \Federator\Data\ActivityPub\Common\APObject[]
     */
    public static function getFollowersByUser($dbh, $id, $connector, $cache)
    {
        // ask cache
        if ($cache !== null) {
            $followers = $cache->getRemoteFollowersOfUser($id);
            if ($followers !== false) {
                return $followers;
            }
        }
        $followers = [];
        // TODO: check our db

        if ($followers === []) {
            // ask connector for user-id
            $followers = $connector->getRemoteFollowersOfUser($id);
            if ($followers === false) {
                $followers = [];
            }
        }
        // save posts to DB
        if ($cache !== null) {
            $cache->saveRemoteFollowersOfUser($id, $followers);
        }
        return $followers;
    }
}
